// Wiring Firmware
// David A. Mellis
// 21 July 2005

// This file documented with Doxygen.
// See http://www.stack.nl/~dimitri/doxygen/docblocks.html for details.

/// \file
/// \brief A serial protocol for controlling the Wiring board using 
/// the serial line routines in serialline.c.
///
/// API: computer to Wiring board
///
/// P:I10<return>: set digital pin 10 to input - pinMode(10, INPUT)
/// P:O10<return>: set digital pin 10 to output - pinMode(10, OUTPUT)
/// P:R10<return>: read digital pin 10 - digitalRead(10)
/// P:S10<return>: set digital pin 10 - digitalWrite(10, HIGH)
/// P:C10<return>: clear digital pin 10 - digitalWrite(10, LOW)
/// P:P1,128<return>: output pwm of 128 (out of 255) on pwm (analog out) pin 1
/// P:A2<return>: read analog pin 2
///
/// API: Wiring board to computer
///
/// P:H10<return>: digital pin 10 is high
/// P:L10<return>: digital pin 10 is low
/// P:V2,63<return>: analog pin 2 has value 63 (out of 1024)

void pinCommand(char cmd[])
{
	int pin = (cmd[0]) ? atoi(cmd + 1) : 0;

	if (cmd[0] == 'I')
		pinMode(pin, INPUT);
	else if (cmd[0] == 'O')
		pinMode(pin, OUTPUT);
	else if (cmd[0] == 'C')
		digitalWrite(pin, LOW);
	else if (cmd[0] == 'S')
		digitalWrite(pin, HIGH);
	else if (cmd[0] == 'R') {
		// this test has to be backwards or we get the
		// wrong result.  WTF?
		if (digitalRead(pin) == HIGH)
			print("P:L%d\n", pin);
		else
			print("P:H%d\n", pin);
	} else if (cmd[0] == 'P') {
		int i, val = 0;

		for (i = 0; cmd[i] && cmd[i] != ','; i++) {}

		if (cmd[i] == ',')
			val = atoi(cmd + i + 1);

		analogWrite(pin, val);
 
		//char p = strchr(cmd, ',');
		//int val = (p == NULL) ? 0 : atoi(p + 1);
		//analogWrite(pin, val);
	} else if (cmd[0] == 'A') {
		print("P:V%d,%d\n", pin, analogRead(pin));
	} else
		print("P:E:Unknown pin command: '%c'.\n", cmd[0]);
}

// this could all be done in loop(), except older versions of Wiring
// use a Java preprocessor, so we can't declare char *, and we can
// only assign a pointer to an array as a function parameter.
void processLine(char line[])
{
	if (line) {
		if (line[0] == 'P' && line[1] == ':')
			pinCommand(line + 2);

		serialLineDispose(line);
	}
}

void setup()
{
	digitalWrite(48, HIGH);
	beginSerial(9600);
	beginSerialLine();
	printMode(SERIAL);
}

void loop()
{
	processLine(serialLine());
}

// serialEvent has to be in this file due to a quirk in Wiring, so
// we just call serialLineEvent(), which does the real work
void serialEvent()
{
	serialLineEvent();
}

